/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function ASRDF_Writer		//<c><a>protected<d>RDF writer class
(
_namespaceSet,		//<p>string[]<d>Namespace prefixes and URI --optional
_doctypeSet			//<p>string[]<d>Namespace prefixes and URI --optional
){
//================
this.ASRDF_Writer = function		//<m>void<a>private<d>class constructor
(
namespaceSet,		//<p>string[]<d>Namespace prefixes and URI --optional
doctypeSet			//<p>string[]<d>Namespace prefixes and URI --optional
){
this.className		= 'ASRDF_Writer';
this.namespaceSet	= namespaceSet ||
	{
	rdf: this.K.NAMESPACE_SET.rdf
	};
this.doctypeSet	= doctypeSet;
//i12n
this.tripleSet = null;
this.datatyping = this.K.DATA_TYPING.ON;
this.set_rdfTagSet(this.K.RDF.PREFIX);
}//</m>ASRDF_Writer



//================
this.set_rdfTagSet = function		//<m>string<a>public<d>Set up rdfTagSet member
(
prefix		//<p>string<d>RDF namespace prefix
){
if(prefix) prefix += ':';
this.rdfTagSet =
	{
	rdf_tag:			sprintf(this.K.RDF.RDF_TAG, prefix, '%s', '%s', '%s', prefix),
	description_tag:	sprintf(this.K.RDF.DESCRIPTION_TAG, prefix, '%s', '%s', prefix),
	description_about_tag:	sprintf(this.K.RDF.DESCRIPTION_ABOUT_TAG, prefix, prefix, '%s'),
	container_tag:		sprintf(this.K.RDF.CONTAINER_TAG, prefix, '%s', '%s', '%s', prefix, '%s'),
	li_tag:				sprintf(this.K.RDF.LI_TAG, prefix, '%s', '%s', '%s', prefix),
    li_resource_tag:	sprintf(this.K.RDF.LI_RESOURCE_TAG, prefix, '%s'),
	about_at:			sprintf(this.K.RDF.ABOUT_AT, prefix, '%s'),
	resource_at: 		sprintf(this.K.RDF.RESOURCE_AT, prefix, '%s'),
	type_at: 			sprintf(this.K.RDF.TYPE_AT, prefix, this.get_XSDEntity(), '%s'),
	parse_at: 			sprintf(this.K.RDF.PARSE_AT, prefix, '%s')
	};
}//</m>set_rdfTagSet



//================
this.set_datatyping = function		//<m>string<a>public<d>Set up datatyping member
(
typing		//<p>boolean<d>Automatic datatyping for literal
){
this.datatyping = typing;
}//</m>set_datatypingv



//================
this.get_XSDEntity = function		//<m>string<a>public<d>Get XML schema namepsace
(){
return (this.K.RDF.XSD_ENTITY in this.doctypeSet) ? '&' + this.K.RDF.XSD_ENTITY + ';' : this.K.RDF.XSD_NS;
}//</m>get_XSDEntity



//================
this.getOut = function		//<m>string<a>public<d>Get RDF model as string
(
tripleSet		//<p>string<d>Parser tripleSet member
){
this.tripleSet = tripleSet;
var
/*string*/content = '';
for(var k in this.tripleSet){
	var
	/*int*/type = this.tripleSet[k][this.K.SLOT.SUBJECT].type;
	content += this.testResourceType(type) ? this.getSubjectDescription(k) : this.getSubjectContainer(k, type);
	}
return this.K.XML.XML_PI + this.getDoctype() + this.getRDF(content);
}//</m>getOut



//================
this.getSubjectDescription = function		//<m>string<a>private<d>Get RDF description element for subject resource
(
subject			//<p>string<d>Subject resource
){
var
/*string*/content = '';
for(var k in this.tripleSet[subject]) if(k != this.K.SLOT.SUBJECT) content += this.getPredicate(subject, k);
return this.getDescription(subject, content);
}//</m>getSubjectDescription



//================
this.getSubjectContainer = function		//<m>string<a>private<d>Get RDF container element for subject resource
(
subject,		//<p>string<d>Subject resource
type			//<p>int<d>Container type
){
var
/*string*/content = '',
/*string*/predicate = this.K.SLOT.CONTAINER_PREDICATE;
for(var k in this.tripleSet[subject][predicate]){
	if(k != this.K.SLOT.PREDICATE){
		var
		/*Object*/outlineSlot = this.tripleSet[subject][predicate][k],
		/*string*/resource = this.getContainerResource(outlineSlot.resource);
		content += (this.testResourceType(outlineSlot.type) ||
			this.testContainerType(outlineSlot.type) ||
			this.testCollectionType(outlineSlot.type)) ?
			this.getContainerMemberResource(this.getResourceAttribute(resource)) :
				this.getContainerMemberLiteral(outlineSlot.literalSet);
		}
	}
return this.getContainer(subject, content);
}//</m>getSubjectContainer



//================
this.getPredicate = function		//<m>string<a>private<d>Get RDF predicate element content
(
subject,		//<p>string<d>Subject resource
predicate		//<p>string<d>Predicate resource
){
var
/*string*/content = '',
/*string*/collection = '',
/*boolean*/collecting = this.testCollectionType(this.tripleSet[subject][predicate][this.K.SLOT.PREDICATE].type);
for(var k in this.tripleSet[subject][predicate]){
	if(k != this.K.SLOT.PREDICATE){
		var
		/*Object*/outlineSlot = this.tripleSet[subject][predicate][k],
		/*string*/resource = outlineSlot.resource,
		/*int*/type = outlineSlot.type;
		if(this.testLiteralType(type)){
			var
			/*string*/method = this.testParseLiteralType(outlineSlot.literalSet.type) ? 'getPredicateParseLiteral' : 'getPredicateLiteral';
			content += this[method](predicate, outlineSlot.literalSet);
			}
		else{
			if(!this.testParseCollectionType(resource)){
				resource = this.getContainerResource(resource);
				if(collecting) collection += this.getDescriptionAbout(resource);
				else content += this.getPredicateResource(predicate, this.getResourceAttribute(resource));
				}
			}
		}
	}
if(collecting) content += this.getPredicateCollection(predicate, collection);
return content;
}//</m>getPredicate



//================
this.getContainerResource = function		//<m>string<a>private<d>Get resource without container id
(
resource		//<p>string<d>Container resource
){
var
/*string[]*/resourceSet = resource.match(this.K.RSC_PATTERN.CONTAINER);
return resourceSet? resourceSet[2] : resource;
}//</m>getContainerResource



//================
this.getDoctype = function		//<m>string<a>private<d>Get DOCTYPE and entity set
(){
if(!this.doctypeSet) return '';
var
/*string*/system = (this.K.RDF.RDF in this.doctypeSet) ? sprintf(this.K.XML.DOCTYPE_SYSTEM, this.doctypeSet[this.K.RDF.RDF]) : '',
/*string*/entities = '';
for(var k in this.doctypeSet) if(k != this.K.RDF.RDF) entities += sprintf(this.K.XML.ENTITY, k, this.doctypeSet[k]);
return (entities || system) ? sprintf(this.K.XML.DOCTYPE, system, entities) : '';
}//</m>getDoctype



//================
this.getRDF = function		//<m>string<a>private<d>Get RDF element and content
(
content		//<p>string<d>RDF element content
){
var
/*string*/namespace = (this.K.RDF.XML_NS in this.namespaceSet) ? this.namespaceSet[this.K.RDF.XML_NS] : '';
return sprintf(this.rdfTagSet.rdf_tag, namespace, this.getNamespaceAttributeSet(), content);
}//</m>getRDF



//================
this.getDescription = function		//<m>string<a>private<d>Get RDF Description element and content
(
resource,		//<p>string<d>Description resource
content			//<p>string<d>RDF content
){
return sprintf(this.rdfTagSet.description_tag, this.getAboutAttribute(resource), content);
}//</m>getDescription



//================
this.getDescriptionAbout = function		//<m>string<a>private<d>Get RDF Description element and content
(
resource		//<p>string<d>Description resource
){
return sprintf(this.rdfTagSet.description_about_tag, resource);
}//</m>getDescriptionAbout



//================
this.getContainer = function		//<m>string<a>private<d>Get RDF container element and content
(
resource,		//<p>string<d>Container resource
content			//<p>string<d>RDF content
){
var
/*string[]*/resourceSet = resource.match(this.K.RSC_PATTERN.CONTAINER);
return resourceSet? sprintf(this.rdfTagSet.container_tag, resourceSet[1], this.getAboutAttribute(resourceSet[2]), content, resourceSet[1]) : '';
}//</m>getContainer



//================
this.getPredicateCollection = function		//<m>string<a>private<d>Get RDF predicate element and literal with collection parseType
(
predicate,		//<p>string<d>Predicate resource
collection		//<p>string<d>Collection content
){
return sprintf(this.K.RDF.PREDICATE_TAG, predicate, this.getParseAttribute(this.K.RDF.PARSE_COLLECTION), '', collection, predicate);
}//</m>getPredicateCollection



//================
this.getPredicateResource = function		//<m>string<a>private<d>Get RDF predicate element and resource
(
predicate,	//<p>string<d>Predicate resource
resource	//<p>string<d>Resource URI
){
return sprintf(this.K.RDF.PREDICATE_RESOURCE_TAG, predicate, resource);
}//</m>getPredicateResource



//================
this.getPredicateLiteral = function		//<m>string<a>private<d>Get RDF predicate element and literal
(
predicate,		//<p>string<d>Predicate resource
literalSet		//<p>Object<d>Literal object
){
return sprintf(this.K.RDF.PREDICATE_TAG, predicate, this.getTypeAttribute(literalSet.type), this.getLangAttribute(literalSet.lang), this.getCData(literalSet.resource), predicate);
}//</m>getPredicateLiteral



//================
this.getPredicateParseLiteral = function		//<m>string<a>private<d>Get RDF predicate element and literal with literal parseType
(
predicate,		//<p>string<d>Predicate resource
literalSet		//<p>Object<d>Literal object
){
return sprintf(this.K.RDF.PREDICATE_TAG, predicate, this.getParseAttribute(literalSet.type), '', literalSet.resource, predicate);
}//</m>getPredicateParseLiteral



//================
this.getContainerMemberResource = function		//<m>string<a>private<d>Get RDF container list element and resource
(
resource	//<p>string<d>Member resource
){
return sprintf(this.rdfTagSet.li_resource_tag, resource);
}//</m>getContainerMemberResource



//================
this.getContainerMemberLiteral = function		//<m>string<a>private<d>Get RDF container list element and literal
(
literalSet		//<p>Object<d>Member literal descriptor
){
return sprintf(this.rdfTagSet.li_tag, this.getTypeAttribute(literalSet.type), this.getLangAttribute(literalSet.lang), this.getCData(literalSet.resource));
}//</m>getContainerMemberLiteral



///================
this.getNamespaceAttributeSet = function		//<m>string<a>private<d>Get RDF namespace attribute set
(){
var
/*string*/namespace = '';
for(var k in this.namespaceSet) if(k != this.K.RDF.XML_NS) namespace += sprintf(this.K.RDF.NS_AT, k, this.namespaceSet[k]);
return namespace;
}//</m>getNamespaceAttributeSet



//================
this.getAboutAttribute = function		//<m>string<a>private<d>Get RDF about attribute
(
resource	//<p>string<d>Resource URI
){
return sprintf(this.rdfTagSet.about_at, resource);
}//</m>getAboutAttribute



//================
this.getResourceAttribute = function		//<m>string<a>private<d>Get RDF resource attribute
(
resource	//<p>string<d>Resource URI
){
return sprintf(this.rdfTagSet.resource_at, resource);
}//</m>getResourceAttribute



//================
this.getTypeAttribute = function		//<m>string<a>private<d>Get RDF type attribute
(
type	//<p>string<d>Type URI
){
return this.datatyping ? sprintf(this.rdfTagSet.type_at, type) : '';
}//</m>getTypeAttribute



//================
this.getParseAttribute = function		//<m>string<a>private<d>Get RDF parseType attribute
(
type	//<p>string<d>Type value
){
return sprintf(this.rdfTagSet.parse_at, type);
}//</m>getParseAttribute



//================
this.getLangAttribute = function		//<m>string<a>private<d>Get XML lang attribute
(
lang	//<p>string<d>Lang code
){
return lang ? sprintf(this.K.XML.LANG_AT, lang) : '';
}//</m>getLangAttribute



//================
this.getCData = function		//<m>string<a>private<d>Get RDF predicate element and literal
(
literal		//<p>Object<d>Literal object
){
return (literal.indexOf(this.K.XML.CDATA) == 0) ? sprintf(this.K.RDF.CDATA, literal.substr(this.K.XML.CDATA_LENGTH)) : literal;
}//</m>getCData



//================
this.testResourceType = function		//<m>boolean<a>private<d>Check whether resource type is one of resources
(
type		//<p>int<d>Resource type
){
return type & this.K.RSC_TYPE.RESOURCE;
//return type < this.K.RSC_TYPE.LITERAL;
}//</m>testResourceType



//================
this.testContainerType = function		//<m>boolean<a>private<d>Check whether resource type is one of container
(
type		//<p>int<d>Resource type
){
return type & (this.K.RSC_TYPE.BAG | this.K.RSC_TYPE.SEQ  | this.K.RSC_TYPE.ALT);
//return (type > this.K.RSC_TYPE.RESOURCE) && (type < this.K.RSC_TYPE.COLLECTION);
}//</m>testContainerType



//================
this.testCollectionType = function		//<m>boolean<a>private<d>Check whether resource type is parse collection
(
type		//<p>int<d>Resource type
){
return type & this.K.RSC_TYPE.COLLECTION;
//return type == this.K.RSC_TYPE.COLLECTION;
}//</m>testCollectionType



//================
this.testLiteralType = function		//<m>boolean<a>private<d>Check whether resource type is one of literal
(
type		//<p>int<d>Resource type
){
return type & (this.K.RSC_TYPE.LITERAL | this.K.RSC_TYPE.MULTILINE);
//return type >= this.K.RSC_TYPE.LITERAL;
}//</m>testLiteralType



//================
this.testParseCollectionType = function		//<m>boolean<a>private<d>Check whether resource is parseCollection
(
type		//<p>string<d>Resource type
){
return type == this.K.RDF.PARSE_COLLECTION;
}//</m>testParseCollectionType



//================
this.testParseLiteralType = function		//<m>boolean<a>private<d>Check whether resource is parseLiteral
(
type		//<p>string<d>Resource type
){
return type == this.K.RDF.PARSE_LITERAL;
}//</m>testParseLiteralType



this.K =
	{
	SLOT:
		{
		SUBJECT:			'<subject>',
		PREDICATE:			'<predicate>',
		CONTAINER_PREDICATE:'_',
		BNODE:				':bnode:'
		},
	NAMESPACE_SET:
		{
		rdf:				'http://www.w3.org/1999/02/22-rdf-syntax-ns#'
		},

	RSC_TYPE:
		{
		UNKNOWN:			0,
		BNODE:				1 << 0,
		RESOURCE:			1 << 1,
		BAG:				1 << 2,
		SEQ:				1 << 3,
		ALT:				1 << 4,
		COLLECTION:			1 << 5,
		LITERAL:			1 << 16,
		MULTILINE:			1 << 17
		},

	DATA_TYPING:
		{
		ON:					true,
		OFF:				false
		},

	XML:
		{
		XML_PI:				'<?xml version="1.0"?>\n',
		DOCTYPE:			'<!DOCTYPE RDF %s\n[%s\n]>\n',
		DOCTYPE_SYSTEM:		' SYSTEM "%s"',
		ENTITY:				'\n<!ENTITY %s "%s">',
		LANG_AT:			' xml:lang="%s"',
		CDATA:				'CDATA:',
		CDATA_LENGTH:		6
		},
	RDF:
		{
		RDF:				'RDF',
		PREFIX:				'rdf',
		PARSE_COLLECTION:	'Collection',
		PARSE_LITERAL:		'Literal',

		XML_NS:				'xmlns',
		XSD_NS:				'http://www.w3.org/2001/XMLSchema#',
		XSD_ENTITY:			'NS.XSD',
		RDF_NS:				'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
		RDF_TAG:			'<%sRDF\n\txmlns="%s"%s>\n%s</%sRDF>\n',
		DESCRIPTION_TAG:	'\n\t<%sDescription%s>\n%s\t</%sDescription>\n',
		DESCRIPTION_ABOUT_TAG:	'\n\t\t\t<%sDescription %sabout="%s" />',
		CONTAINER_TAG:		'\n\t<%s%s%s>\n%s\t</%s%s>\n',
		LI_TAG:				'\t\t<%sli%s%s>%s</%sli>\n',
		LI_RESOURCE_TAG:	'\t\t<%sli %s />\n',
		PREDICATE_TAG:		'\t\t<%s%s%s>%s</%s>\n',
		PREDICATE_RESOURCE_TAG:		'\t\t<%s %s />\n',

		NS_AT:				'\n\txmlns:%s="%s"',
		ABOUT_AT:			' %sabout="%s"',
		RESOURCE_AT: 		' %sresource="%s"',
		TYPE_AT: 			' %sdatatype="%s%s"',
		PARSE_AT: 			' %sparseType="%s"',
		CDATA:				'<![CDATA[%s]]>'
		},

	RSC_PATTERN:
		{
		CONTAINER:			/^\((Bag|Seq|Alt)\)\s*(.*)$/
		}
	};



this.ASRDF_Writer(_namespaceSet, _doctypeSet);
}//</c>ASRDF_Writer



