/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function DIGEST_Generator		//<c>abstract<a>public<d>Abstract encryption class
(
_hexCase,		//<p>boolean=false<d>Hexadecimal output case flag: uppercase || lowercase --optional
_charSize		//<p>int=8<d>Message character size --optional (8 bits)
){
//================
this.DIGEST_Generator = function		//<m>void<a>private<d>class constructor
(
hexCase,		//<p>boolean=false<d>Hexadecimal output case flag: uppercase || lowercase --optional
charSize		//<p>int=8<d>Message character size --optional (8 bits)
){
this.className	= 'DIGEST_Generator';
const
/*boolean*/LOWER_CASE = false,
/*int*/BYTE_SIZE = 8;
this.hexCase	= hexCase || LOWER_CASE;
this.charSize	= charSize || BYTE_SIZE;
//i12n
this.b64Pad		= '';
this.charMask	= (1 << this.charSize) - 1;
this.hexTab		= this.hexCase ? UPPER_HEX : LOWER_HEX;
}//</m>DIGEST_Generator



//================
this.getWordSetFromString = function		//<m>int[]<a>private<d>Convert a string to an array of little-endian words
(
msg		//<p>string<d>Message to convert
){
var
/*int[]*/wordSet = [];
for(var i=0, m=msg.length * this.charSize; i<m ; i+=this.charSize) wordSet[i>>5] |= (msg.charCodeAt(i / this.charSize) & this.charMask) << (i%32);
return wordSet;
}//</m>getWordSetFromString<d>&
/*
If charSize member is 8-bit (ASCII), characters > 255 have their hi-byte silently ignored.
*/



//================
this.getHexFromWordSet = function		//<m>string<a>private<d>Convert an array of little-endian words to a hex string
(
wordSet		//<p>int[]<d>Word array
){
var
/*string*/str = '';
for(var i=0, m=wordSet.length * 4; i<m; i++) str += this.hexTab.charAt((wordSet[i>>2] >> ((i%4)*8+4)) & 0xF) +
	this.hexTab.charAt((wordSet[i>>2] >> ((i%4)*8)) & 0xF);
return str;
}//</m>getHexFromWordSet



//================
this.getBase64FromWordSet = function		//<m>string<a>private<d>Convert an array of little-endian words to a hex string
(
wordSet		//<p>int[]<d>Word array
){
var
/*string*/str = '';
for(var i=0, m=wordSet.length * 4; i<m; i+=3){
	var
	/*int*/triplet = (((wordSet[i >> 2] >> 8 * ( i%4)) & 0xFF) << 16)
		| (((wordSet[i+1 >> 2] >> 8 * ((i+1)%4)) & 0xFF) << 8 )
		|  ((wordSet[i+2 >> 2] >> 8 * ((i+2)%4)) & 0xFF);
	for(var j=0; j<4; j++) str += ((i * 8 + j * 6) > (wordSet.length * 32)) ?
		this.b64Pad : BASE64.charAt((triplet >> 6*(3-j)) & 0x3F);
	}
return str;
}//</m>getBase64FromWordSet



//================
this.getStringFromWordSet = function		//<m>string<a>private<d>Convert an array of little-endian words to a string
(
wordSet		//<p>int[]<d>Word array
){
var
/*string*/str = '';
for(var i=0, m=wordSet.length * 32; i<m; i+=this.charSize) str += String.fromCharCode((wordSet[i>>5] >>> (i % 32)) & this.charMask);
return str;
}//</m>getStringFromWordSet



//================
this.getBitRotation = function		//<m>int<a>private<d>Bitwise rotate a 32-bit number to the left
(
int32,		//<p>int<d>Number to rotate
rotation	//<p>int<d>Rotation count
){
return (int32 << rotation) | (int32 >>> (32 - rotation));
}//</m>getBitRotation



//================
this.getSafeAdd = function		//<m>int<a>private<d>Add integers, wrapping at 2^32
(
op1,		//<p>int<d>First operand
op2			//<p>int<d>Second operand
){
var lsw = (op1 & 0xFFFF) + (op2 & 0xFFFF);
return (((op1 >> 16) + (op2 >> 16) + (lsw >> 16)) << 16) | (lsw & 0xFFFF);
}//</m>getSafeAdd<d>&
/*
This uses 16-bit operations internally to work around bugs in some JS interpreters
*/



//================
this.getStringFromHex = function		//<m>string<a>public<d>Convert an number to hexa representation
(
hexval			//<p>int<d>Hexadecimal value
){
var
str = '';
for(var i=0, m=8; i<m; i++) str = this.hexTab[(hexval >>> (i*4)) & 0xF] + str;
return str;
}//</m>getStringFromHex



//================
this.testMessage = function		//<m>boolean<d>private<d>Check whether message to digest is not empty
(
msg		//<p>string<d>Message to digest
){
return (msg != '') && (msg != undefined) && (msg != null);
}//</m>testMessage



const /*string*/
UPPER_HEX	= '0123456789ABCDEF',
LOWER_HEX	= '0123456789abcdef',
BASE64		= 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';



this.DIGEST_Generator(_hexCase, _charSize);
}//</c>DIGEST_Generator

