/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function IO_file_dirHandler		//<c><a>protected<d>Directory handler class
(
_instantiator		//<p>IO_file_Instantiator<d>File mediator instance
){
//================
this.IO_file_dirHandler = function		//<m>void<a>private<d>class constructor
(
instantiator		//<p>IO_file_Instantiator<d>File mediator instance
){
this.className		= 'IO_file_dirHandler';
this.instantiator	= instantiator;
this.setIn();
}//</m>IO_file_dirHandler



//================
this.setIn = function		//<m>void<a>private<d>Instance i12n
(){
this.errorNotifier	= new ERROR_Notifier(this.className);
this.mode			= null;
}//</m>setIn



//================
this.set_mode = function		//<m>int<a>protected<d>Setup mode member
(
mode		//<p>string=RDWR_MODE<d>Mode code --optional
){
this.mode = mode || this.K.RDWR_MODE;
}//</m>set_mode



//================
this.setOut = function		//<m>int<a>public<d>Create a set of new directories
(
outputSet,		//<p>string[]<d>Names of directory to create
permissions		//<p>int<d>Permission code
){
var
/*int*/failures = 0,
/*int*/capacity = outputSet.length,
/*boolean||string*/lastItemAsNesting = outputSet[capacity -1],
/*boolean*/nesting = lastItemAsNesting === true;
if(nesting) capacity--;		//strip last item, i.e. nesting flag
for(var i=0;i<capacity;i++) failures += this.setOutMake(nesting ? this.handle : this.handle.clone(), outputSet[i], permissions);
return failures;
}//</m>setOut<d>&
/*
setOut method creates a set of directories inside the handled directory, either:
at the same level (default) or
nested from first to last, if the last item of the set is a true boolean flag
*/



//================
this.setOutMake = function		//<m>int<a>private<d>Create a new directory
(
handle,			//<p>XPCOM_nsIFile<d>Directory handle
leaf,			//<p>string[]<d>Name of directory to create
permissions		//<p>int<d>Permission code
){
try{
	handle.append(leaf);
	handle.create(this.K.DIR_INSTANCE, permissions);
	return this.K.ERROR.NONE;
	}
catch(e){
	this.errorNotifier.setOut(e + ' @' + leaf + '\n', this.K.ERROR.MAKE_DENY);
	return this.K.ERROR.NO_SUCCESS;
	}
}//</m>setOutMake<d>



//================
this.get_mode = function		//<m>int<a>protected<d>Get mode member
(){
return this.mode;
}//</m>get_mode



//================
this.getIn = function		//<m>IO_DirDescriptor<a>public<d>Get the content of the handled directory
(
profile,		//<p>int<d>Type of content collected: files, directories, symlinks, hidden... --optional
pattern			//<p>string<d>Pattern for files/directories to collect, converted to a RegExp --optional
){
try{
	var
	/*int*/at = 0,
	/*Object*/item,
	/*IO_DirDescriptor*/itemSet =
		{
		fileHandleSet:	[],
		dirHandleSet:	[]
		},
	/*Object*/conditionSet = this.getCollectingConditionSet(arguments),
	/*Object*/iterator = this.handle.directoryEntries;
	while(iterator.hasMoreElements()){
		item = XPConnector.getQuery('nsIFile', iterator.getNext());
		if((this.testCollectByProfile(item, conditionSet.profile)) &&
		(this.testCollectByPattern(item, conditionSet.pattern))){
			itemSet[item.isDirectory() ? 'dirHandleSet' : 'fileHandleSet'][item.path] = item;
			}
		}
	return new IO_file_DirDescriptor(/*this.instantiator, */itemSet);
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.DIR_COLLECTING);
	return null;
	}
}//</m>getIn<d>&
/*
getIn collects the content of the handled directory
Collection can be restricted to files or directories,
eventualy matching a specific pattern (RegExp)
*/



//================
this.getCollectingConditionSet = function		//<m>Object<a>private<d>Get conditions fot directory listing
(
paramSet		//<p>any[]<d>getIn method arguments
){
var
/*Object*/conditionSet =
	{
	profile:	(paramSet[0] != undefined) ? paramSet[0] : this.K.COLLECT_ALL,
	pattern: 	(paramSet[1] != undefined) ? new RegExp(paramSet[1]) : null
	};
return conditionSet;
}//</m>getCollectingConditionSet



//================
this.testCollectByProfile = function		//<m>boolean<a>private<d>Check whether the item type conforms to the collected type
(
item,			//<p>XPCOM_nsIFile<d>Directory entry
profile			//<p>int<d>Collection type flag
){
var
/*int*/itemProfile 	= 0;
/*boolean*/isFile	= item.isFile(),
/*boolean*/isDir	= item.isDirectory(),
/*boolean*/isLink	= item.isSymlink();
if(item.isHidden()){
	if(isFile) itemProfile |= this.K.COLLECT_HIDDEN_FILE;
	else{
		if(isDir) itemProfile |= this.K.COLLECT_HIDDEN_DIR;
		else itemProfile |= this.K.COLLECT_HIDDEN_LINK;
		}
	}
else{
	if(isFile) itemProfile |= this.K.COLLECT_FILE;
	else{
		if(isDir) itemProfile |= this.K.COLLECT_DIR;
		else itemProfile |= this.K.COLLECT_LINK;
		}
	}
return ((profile & itemProfile) != 0);
}//</m>testCollectByProfile



//================	FIXME: TO BE IMPLEMENTED
this.testCollectByProfile1 = function		//<m>boolean<a>private<d>Check whether the item type conforms to the collected type
(
item,			//<p>XPCOM_nsIFile<d>Directory entry
profile			//<p>int<d>Collection type flag
){
var
/*int*/itemProfile 	= 0;
if(item.isFile()) itemProfile |= this.K.COLLECT_FILE;
else{
	if(item.isDirectory()) itemProfile |= this.K.COLLECT_DIR;
	else if(item.isSymlink()) itemProfile |= this.K.COLLECT_LINK;
	}
if(item.isHidden()) itemProfile |= this.K.COLLECT_HIDDEN;
else itemProfile |= this.K.COLLECT_NOT_HIDDEN;
if(item.isReadable()) itemProfile |= this.K.COLLECT_READABLE;
else itemProfile |= this.K.COLLECT_NOT_READABLE;
if(item.isWritable()) itemProfile |= this.K.COLLECT_WRITABLE;
else itemProfile |= this.K.COLLECT_NOT_WRITABLE;
if(item.isExecutable()) itemProfile |= this.K.COLLECT_EXECUTABLE;
else itemProfile |= this.K.COLLECT_NOT_EXECUTABLE;
return ((profile & itemProfile) != 0);
}//</m>testCollectByProfile



//================
this.testCollectByPattern = function		//<m>boolean<a>private<d>Check whether the item pattern conforms to the collected pattern
(
item,			//<p>XPCOM_nsIFile<d>Directory entry
pattern			//<p>int<d>Pattern flag of items allowed
){
return (!pattern || (item.path.match(pattern) != null));
}//</m>testCollectByPattern



this.K =
	{
	FILE_INSTANCE:				0,				//synchronise with XPCOM_nsIFile NORMAL_FILE_TYPE
	DIR_INSTANCE:				1,				//synchronise with XPCOM_nsIFile DIRECTORY_TYPE
	
	COLLECT_FILE:				1 << 0,
	COLLECT_DIR:				1 << 1,
	COLLECT_LINK:				1 << 2,
	//REFACTOR: REPLACED
	COLLECT_HIDDEN_FILE:		1 << 3,
	COLLECT_HIDDEN_DIR:			1 << 4,
	COLLECT_HIDDEN_LINK:		1 << 5,
	//REFACTOR: REPLACING
	COLLECT_NOT_HIDDEN:			1 << 8,
	COLLECT_HIDDEN:				1 << 9,
	COLLECT_NOT_READABLE:		1 << 10,
	COLLECT_READABLE:			1 << 11,
	COLLECT_NOT_WRITABLE:		1 << 12,
	COLLECT_WRITABLE:			1 << 13,
	COLLECT_NOT_EXECUTABLE:		1 << 14,
	COLLECT_EXECUTABLE:			1 << 15,
	COLLECT_ALL:				0xFFFFFFFF,
	
	//DEEPER_COLLECT:				0x0FFFFFFF,		//should be enougth!		FIXME: TO BE IMPLEMENTED

	ERROR:
		{
		//NON FATAL
		NONE:					0,
		NO_SUCCESS:				1,

		//FATAL
		MAKE_DENY:				-1,
		DIR_COLLECTING:			-2,
		FATAL:					0xFF		//end of error codes
		}
	};



this.IO_file_dirHandler(_instantiator);
}//</c>IO_file_dirHandler<d>&
/*
This component class manages handled dir operations:
	input (read) to list directory content,
	output (write) to create other directories
*/

