/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function IO_file_Processor		//<c><a>protected<d>Process running class
(
_instantiator,		//<p>IO_file_processInstantiator<d>File process instance
_argumentSet,		//<p>string[]<d>Processing arguments --optional if no _pid, otherwise null
_pid				//<p>int<d>Process ID
){
//================
this.IO_file_Processor = function		//<m>void<a>private<d>class constructor
(
instantiator,		//<p>IO_file_Instantiator<d>File mediator instance
argumentSet,		//<p>string[]<d>Process arguments --optional if no pid, otherwise null
pid					//<p>int<d>Process ID --optional
){
this.className		= 'IO_file_Processor';
this.instantiator	= instantiator;
this.argumentSet	= argumentSet;
this.pid			= pid;
this.setIn();
}//</m>IO_file_Processor



//================
this.setIn = function		//<m>Instance i12n
(){
this.errorNotifier	= new ERROR_Notifier(this.className);
this.runCode 		= null;
this.killCode 		= null;
this.contextID		= this.K.START_CONTEXT;
this.blocking		= this.K.PROCESS_NON_BLOCKING;
this.contextSet		= [];
this.enable			= this.setUp();
}//</m>setIn



//================
this.setUp = function		//<m>Instance i12n
(){
try{
	this.process = this.instantiator.process;
	if(this.pid) this.process.initWithPid(this.pid);
	else this.process.init(this.instantiator.handle);
	return this.K.INSTANCE_ENABLED;
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.PROCESS_INIT);
	return this.K.INSTANCE_DISABLED;
	}
}//</m>setUp



//================
this.set_blocking = function		//<m>void<a>public<d>Set up blocking member
(
blocking		//<p>boolean<d>Blocking flag: synchronous : asynchronous
){
this.blocking = blocking;
}//</m>set_blocking



//================
this.set_contextSet = function		//<m>void<a>public<d>Set up blocking flag member
(
contextSet		//<p>string[]<d>Environment variable identifiers --optional
){
var
/*Object*/context =
	{
	//pid:		this.process.pid,					//disabled properties are NOT IMPLEMENTED in Mozilla
	//file:		this.process.location,
	//name:		this.process.processName,
	//signature:	this.process.processSignature,
	//exit:		this.process.exitValue,
	run:		this.runCode,
	kill:		this.killCode
	}
//if(contextSet) for(var i=0,m=contextSet.length;i<m;i++) context[contextSet[i]] = this.process.getEnvironment(contextSet[i]);	NOT IMPLEMENTED
this.contextSet[this.contextID++] = context;
}//</m>set_contextSet



//================
this.setProcess = function		//<m>IO_file_Processor<a>protected<d>Execute (run / kill) the process
(
killing,		//<p>boolean<d>Kill process flag: kill || run --optional
contextSet		//<p>string[]<d>Environment variables identifiers to be saved after run command --optional
){
if(!this.testEnable()) return this.K.NO_SUCCESS;
try{
	if(!killing){
		var
		/*string*/argSet = this.argumentSet || null,
		/*int*/argCount = argSet ? this.argumentSet.length : null,
		/*Object*/pid = this.K.PID_OBJECT;
		this.runCode = this.process.run(this.blocking, argSet, argCount, pid);
		}
	else this.killCode = this.process.kill();
	this.set_contextSet(contextSet);
	return this;
	}
catch(e){
	return this.errorNotifier.setOut(e, killing ? this.K.ERROR.PROCESS_KILL : this.K.ERROR.PROCESS_RUN);
	}
}//</m>setProcess<d>&
/*
Method returns its class instance to eventually retrieve the process environment variables saved after run command
*/



//================
this.getContextAt = function		//<m>void<a>public<d>Get context member
(
at		//<p>int<d>Context set index to get
){
return this.contextSet[at];
}//</m>getContextAt



//================
this.testEnable = function		//<m>boolean<a>private<d>Check whether enable member is set
(){
return this.enable;
}//</m>testEnable



this.K =
	{
	INSTANCE_ENABLED:			true,
	INSTANCE_DISABLED:			false,
	PROCESS_NON_BLOCKING:		0,
	PROCESS_BLOCKING:			1,
	START_CONTEXT:				0,
	PID_OBJECT:					{ },		//run method NOT IMPLEMENTED

	ERROR:
		{
		//NON FATAL
		NONE:					0,
		NO_SUCCESS:				1,

		//FATAL
		PROCESS_INIT:			-1,
		PROCESS_RUN:			-2,
		PROCESS_KILL:			-3,
		FATAL:					0xFF		//end of error codes
		}
	};



this.IO_file_Processor(_instantiator, _argumentSet, _pid);
}//</c>IO_file_Processor



