/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function RDFAPI_Collector		//<c><a>public<d>RDF container collections class
(
_source,		//<p>Object<d>Resource node or URN or literal
_type,			//<p>int<d>Collection type
_store			//<p>RDFAPI_Store<d>Collector store
){
const ASSERTING = false;
//================
this.RDFAPI_Collector = function		//<m>void<a>private<d>class constructor
(
source,			//<p>Object<d>Resource node or URN or literal
type,			//<p>int<d>Collection type
store			//<p>RDFAPI_Store<d>Collector store
){
this.className	= 'RDFAPI_Collector';
this.source		= source;
this.type		= type;
this.store		= store;
//i12n
this.errorNotifier = new ERROR_Notifier(this.className);
if(this.setXPCOM() == this.K.ERROR.NONE){
	this.renumbering = this.K.RENUMBER_COLLECTION;
	this.set_node();
	this.set_collection();
	}
}//</m>RDFAPI_Collector



//================
this.setXPCOM = function		//<m>void<a>private<d>SetUp RDF service interface
(){
try{
	this.XPCOM_RDFContainerUtils = XPConnector.getService('nsIRDFContainerUtils');
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.SERVICE_ACCESS);
	}
}//</m>setXPCOM



//================
this.set_node = function		//<m>void<a>private<d>Set up the collection node member
(){
this.node = (this.source instanceof RDFAPI_Resource) ? this.source :
	new RDFAPI_Resource(this.source, this.getSourceType());
}//</m>set_node



//================
this.set_collection = function		//<m>Object<a>private<d>Set up the collection member
(){
var
/*string*/method = (this.type == this.K.SEQ_COLLECTION) ? 'MakeSeq' :
		(this.type == this.K.BAG_COLLECTION) ? 'MakeBag' :
			'MakeAlt';
this.collection = this.getService(method, this.K.ERROR.COLLECTION_MAKE);
}//</m>set_collection



//================
this.setAddItem = function		//<m>Object<a>public<d>Add a resource to the collection
(
item,		//<p>Object<d>Resource to add: statement or collection
at			//<p>int<d>1-based index to insert item -- optional: append
){
if(this.testCollectable(item)){
	try{
		var
		/*object*/node = this.getItemResource(item).getAsNode();
		if(at) this.collection.InsertElementAt(node, at, this.renumbering);
		else this.collection.AppendElement(node, this.renumbering);
		return item;
		}
	catch(e){
		this.errorNotifier.setOut(e, this.K.COLLECTION_ADD);
		}
	}
return null;
}//</m>setAddItem



//================
this.setRemoveItem = function		//<m>RDFAPI_Resource<a>public<d>Remove a resource from the collection, returning it
(
item,		//<p>RDFAPI_Resource<d>Resource to remove
at			//<p>int<d>1-based index to remove item -- optional
){
try{
	if(at){
		if(this.testInRange(at)){
			this.collection.RemoveElementAt(at, this.renumbering);
			return item;
			}
		}
	else{
		if(this.testCollectable(item)){
			this.collection.RemoveElement(this.getItemResource(item).getAsNode(), this.renumbering);	//FIXME: getAsNode
			return item;
			}
		}
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.COLLECTION_REMOVE);
	}
return null;
}//</m>setRemoveItem



//================
this.setItemSet = function		//<m>int<a>public<d>Add in / Remove from the collection resources from an object or array
(
itemSet,		//<p>Object<d>Provider object
adding,			//<p>boolean=false<d>Add flag: add || remove --optional
at,				//<p>int<d>1-based index to add / remove first item -- optional
collectedSet	//<p>Object[]<d>Added / removed items collector -- optional
){
var
/*Object[]*/collectedSet = collectedSet || [],
/*string*/	method = adding ? 'setAddItem' : 'setRemoveItem',
/*int*/		startCount = this.getLength();
for(var k in itemSet) collectedSet.push(this[method](itemSet[k], at ? at++ : at));	//DO NOT increment without renumbering, to avoid index out of range
var
/*int*/endCount = this.getLength();
return endCount - startCount;		//number of items added / removed
}//</m>setItemSet



//================
this.set_renumbering = function		//<m>boolean<a>public<d>Setup renumbering member, returning previous value
(){
var
/*boolean*/previous = this.renumbering
this.renumbering = renumbering;
return previous;
}//</m>set_renumbering



//================
this.getService = function		//<m>Object<a>private<d>Get RDF container service
(
method,		//<p>string<d>Service method to invoque
code		//<p>int<d>Error code in case of service failure
){
try{
	return this.XPCOM_RDFContainerUtils[method](this.store, this.getResource());
	}
catch(e){
	this.errorNotifier.setOut(e, code);
	return null;
	}
}//</m>getService



//================
this.get_node = function		//<m>int<a>public<d>Get collection RDFAPI_Resource object
(){
return this.node;
}//</m>get_node



//================
this.getSourceType = function		//<m>int<a>private<d>Get the collection resource type
(){
return ((typeof this.source) == 'object') ? this.K.INSTANCE_RESOURCE :
	(this.source == '') ? this.K.ANONYMOUS_RESOURCE :
		this.K.NAMED_RESOURCE;
}//</m>getSourceType



//================
this.getResource = function		//<m>int<a>public<d>Get collection nsIRDFResource object
(){
return this.node.getAsResource();
}//</m>getResource<d>&
/*
Method callable before rsc member i12n
*/



//================
this.getEnumerator = function		//<m>Object<a>public<d>Get a collection enumerator
(){
return this.collection.GetElements();
}//</m>getEnumerator



//================
this.getItemResource = function		//<m>Object<a>private<d>Get a collection item RDFAPI_Resource instance
(
item		//<p>Object<d>Resource to get
){
return (item instanceof RDFAPI_Statement) ? item.getSubject() :
			(item instanceof RDFAPI_Collector) ? item.get_node() :
			null;
}//</m>getItemResource



//================
this.getItemAt = function		//<m>Object<a>public<d>Get the resource item according to a 0-based index (i.e. array indexing)
(
at		//<p>int<d>Item to get index at
){
if(!this.testEmpty()){
	try{
		return this.collection.DataSource.GetTarget(this.getResource(), this.XPCOM_RDFContainerUtils['IndexToOrdinalResource'](at + 1), true);
		}
	catch(e){
		this.errorNotifier.setOut(e, this.K.COLLECTION_RETRIEVE);
		}
	}
return null;
}//</m>getItemAt



//================
this.getIndexOf = function		//<m>Object<a>public<d>Get the item index according to its resource
(
resource	//<p>int<d> Item Resource to get index
){
if(!this.testEmpty()){
	try{
		return this.XPCOM_RDFContainerUtils['OrdinalResourceToIndex'](resource);
		}
	catch(e){
		this.errorNotifier.setOut(e, this.K.COLLECTION_RETRIEVE);
		}
	}
return -1;
}//</m>getIndexOf<d>&
/*
If item is not found, -1 is return (indexOf method behaviour)
*/



//================
this.getLength = function		//<m>int<a>public<d>Get the number of items in this collection
(){
return this.testEmpty() ? 0 : this.collection.GetCount();
}//</m>getLength



//================
this.testCollectionType = function		//<m>boolean<a>public<d>Test the collection type
(){
var
/*string*/method = (this.type == this.K.BAG_COLLECTION) ? 'IsBag' :
		(this.type == this.K.SEQ_COLLECTION) ? 'IsSeq' :
			'IsAlt';
return this.getService(method, this.K.ERROR.COLLECTION_TEST);
}//</m>testCollectionType



//================
this.testEmpty = function		//<m>boolean<a>public<d>Check whether this collector is empty
(){
return this.getService('IsEmpty', this.K.ERROR.COLLECTION_TEST);
}//</m>testEmpty



//================
this.testCollectable = function		//<m>boolean<a>private<d>Check whether an item is insertable in / removable from a collection
(
item		//<p>Object<d>Item to insert / remove
){
try{
	return (item instanceof RDFAPI_Statement) || (item instanceof RDFAPI_Collector);
	}
catch(e){
	return false;
	}
}//</m>testCollectable



//================
this.testInRange = function		//<m>boolean<a>private<d>Check whether the index is in the range of the collection
(
at		//<p>int<d>Index
){
return at <= this.getLength();
}//</m>testInRange



this.K =
	{
	BAG_COLLECTION:				1,
	SEQ_COLLECTION:				2,
	ALT_COLLECTION:				3,

	INSTANCE_RESOURCE:			0,
	NAMED_RESOURCE:				2,
	ANONYMOUS_RESOURCE:			3,

	RENUMBER_COLLECTION:		true,
	
	ERROR:
		{
		//NON FATAL
		NONE:					0,
		NO_SUCCESS:				1,

		//FATAL
		SERVICE_ACCESS:			-1,
		COLLECTION_MAKE:		-3,
		COLLECTION_TEST:		-4,
		COLLECTION_ADD:			-5,
		COLLECTION_REMOVE:		-6,
		COLLECTION_RETRIEVE:	-7,
		FATAL:					0xFF		//end of error codes
		}
	};



this.RDFAPI_Collector(_source, _type, _store);
}//</c>RDFAPI_Collector

